@@ -1,6 +1,19 @@
 /*
  * Copyright (C) 2012, 2015 IBM Corporation
- * See 'COPYRIGHT' for License of this code.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -182,6 +195,7 @@ bluehawk_list_leds(const char *enclosure, const char *component, int verbose)
 		fprintf(stderr,
 			"%s: cannot read diagnostic page from SES for %s\n",
 			progname, enclosure);
+		close(fd);
 		return -1;
 	}
 
@@ -191,8 +205,11 @@ bluehawk_list_leds(const char *enclosure, const char *component, int verbose)
 		unsigned int cindex;
 		enum bh_component_type ctype;
 		rc = decode_component_loc(component, &ctype, &cindex);
-		if (rc != 0)
+		if (rc != 0) {
+			close(fd);
 			return -1;
+		}
+
 		report_component_from_ses(&dp, ctype, cindex, verbose);
 	} else {
 		unsigned int i;
@@ -217,6 +234,7 @@ bluehawk_list_leds(const char *enclosure, const char *component, int verbose)
 								verbose);
 	}
 
+	close(fd);
 	return 0;
 }
 
