@@ -3,6 +3,19 @@
  *
  * Copyright (C) International Business Machines Corp., 2009
  *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 using namespace std;
@@ -24,6 +37,12 @@ using namespace std;
 #include "catalogs.h"
 #include <sstream>
 
+extern FILE *spopen(char **, pid_t *) __attribute__((weak));
+extern FILE *spclose(FILE *, pid_t)  __attribute__((weak));
+
+//Workaround for deprecated warning
+#pragma GCC diagnostic ignored "-Wwrite-strings"
+
 enum regex_text_policy regex_text_policy = RGXTXT_READ;
 
 static CatalogCopy *catalog_copy = NULL;
@@ -55,7 +74,7 @@ Parser::parse_file(const string& path)
 	int result;
 
 	cur_parser = this;
-	pathname = strdup(path.c_str());
+	pathname = path.c_str();
 	file = fopen(pathname, "r");
 	if (!file) {
 		fprintf(stderr,
@@ -67,7 +86,8 @@ Parser::parse_file(const string& path)
 	result = parse();
 	if (semantic_errors > 0)
 		result = semantic_errors;
-		fclose(file);
+
+	fclose(file);
 	return result;
 }
 
@@ -335,12 +355,13 @@ Parser::get_text_block(void)
 			return NULL;
 	}
 	if (last_non_space < 0)
-	return strdup("");
+		return strdup("");
+
 	strbuf[last_non_space+1] = '\0';
 	return strdup(strbuf);
-	}
+}
 
-	ReporterCtlgParser::ReporterCtlgParser() : Parser() {
+ReporterCtlgParser::ReporterCtlgParser() : Parser() {
 	parse = rrparse;
 	error = rrerror;
 }
@@ -354,9 +375,9 @@ void
 MemberSet::tally(void *addr, const string& name)
 {
 	if (seen.find(addr) == seen.end())
-			seen.insert(addr);
+		seen.insert(addr);
 	else
-	parser->semantic_error(name + " statement seen multiple times"
+		parser->semantic_error(name + " statement seen multiple times"
 						" in same catalog entry.");
 }
 
@@ -487,7 +508,7 @@ Reporter::set_prefix_args(vector<string> *args)
 {
 	members.tally(&prefix_args, "prefix_args");
 	if (!prefix_args)
-	prefix_args = args;
+		prefix_args = args;
 }
 
 bool
@@ -663,9 +684,9 @@ MetaReporter::validate(ReporterCatalog *catalog)
 void
 ReporterCatalog::register_reporter(Reporter* r)
 {
-	if (!r)
-	/* called as a result of a syntax error */
-	return;
+	if (!r) /* called as a result of a syntax error */
+		return;
+
 	rlist.push_back(r);
 	register_alias(r->base_alias, r);
 
@@ -683,6 +704,7 @@ ReporterCatalog::register_meta_reporter(MetaReporter *mr)
 {
 	if (!mr)
 		return;
+
 	mr->validate(this);
 	if (find(mr->name) || find_meta_reporter(mr->name)) {
 		cur_parser->semantic_error(
@@ -865,14 +887,19 @@ SyslogEvent::SyslogEvent(const string& rp, const string& sev, const string& fmt,
 void
 MatchVariant::compute_regex_text(void)
 {
-	int get_prefix_args = 0;
+	int __attribute__((__unused__))get_prefix_args = 0;
 	FILE *in;
 	char regex_cstr[REGEX_MAXLEN];
+	std::ostringstream regex_maxlen;
+	string regex_max_s;
+	char *regex_len, *format;
+	char *args[4];
+	pid_t cpid;
 	Reporter *reporter = reporter_alias->reporter;
 	string full_format = reporter->prefix_format + parent->format;
 	size_t nl = full_format.find_last_of('\n');
 
-	// Strip trailing newline. 
+	// Strip trailing newline.
 	if (nl) {
 		if (full_format.substr(nl+1) != "")
 		parent->parser->semantic_error(
@@ -883,39 +910,53 @@ MatchVariant::compute_regex_text(void)
 	if (reporter->prefix_args && reporter->prefix_args->size() > 0)
 		get_prefix_args = 1;
 
-	std::stringstream command; 
-	command <<  "regex_converter " << REGEX_MAXLEN << " " << "\""; 
-	command << full_format << "\" " << get_prefix_args;
+	regex_maxlen << REGEX_MAXLEN;
+	regex_max_s = regex_maxlen.str();
+	regex_len = strdup(regex_max_s.c_str());
+	format = strdup(full_format.c_str());
+	if ((!regex_len) || (!format)) {
+		parent->parser->semantic_error("Memory allocation failed");
+		goto free_mem;
+	}
 
-	std::string tmp = command.str();
-	const char* cstr = tmp.c_str();
+	args[0] = "/usr/bin/regex_converter";
+	args[1] = regex_len;
+	args[2] = format;
+	args[3] = NULL;
 
-	if (!(in = popen(cstr, "r"))) {
+	if (!(in = spopen(args, &cpid))) {
 		parent->parser->semantic_error("cannot create regex text,"
 				"regex_converter may not be installed");
-		return;
+		goto free_mem;
 	}
- 
+
 	fgets(regex_cstr,REGEX_MAXLEN, in);
- 
-	pclose(in);
+
+	spclose(in, cpid);
 
 	if (!strcmp(regex_cstr, "regex parser failure")) {
 		parent->parser->semantic_error(
 				"cannot create regex text from format");
-		return;
+		goto free_mem;
 	}
 
 	regex_text = regex_cstr;
 
 	nl = regex_text.find_last_of('\n');
 
-	// Strip trailing newline. 
+	// Strip trailing newline.
 	if (nl) {
- 		regex_text = regex_text.substr(0, nl);
+		regex_text = regex_text.substr(0, nl);
 	}
-	// Change expr to ^expr$ so we match only the full message. 
+	// Change expr to ^expr$ so we match only the full message.
 	regex_text = "^" + regex_text + "$";
+
+free_mem:
+	if (regex_len)
+		free(regex_len);
+
+	if (format)
+		free(format);
 }
 
 
@@ -1127,12 +1168,12 @@ SyslogEvent::match(SyslogMessage *msg, bool get_prefix_args)
 
 	vector<MatchVariant*>::iterator it;
 	for (it = match_variants.begin(); it < match_variants.end(); it++) {
-	if ((*it)->match(msg, get_prefix_args)) {
-		matched_variant = *it;
-		break;
+		if ((*it)->match(msg, get_prefix_args)) {
+			matched_variant = *it;
+			break;
+		}
 	}
-}
-return matched_variant;
+	return matched_variant;
 }
 
 int
