@@ -3,7 +3,20 @@
  * @brief	Light Path Diagnostics
  *
  * Copyright (C) 2012 IBM Corporation
- * See 'COPYRIGHT' for License of this code.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * @author	Vasant Hegde <hegdevasant@linux.vnet.ibm.com>
  */
@@ -16,11 +29,11 @@
 #include <errno.h>
 #include <getopt.h>
 #include <curses.h>
-#include <ncurses/menu.h>
+#include <menu.h>
 #include <signal.h>
 
-#include "platform.c"
-#include "lp_diag.h"
+#include "servicelog.h"
+#include "indicator.h"
 #include "lp_util.h"
 
 /* FRU callout priority as defined in PAPR+
@@ -78,9 +91,9 @@ set_attn_indicator(struct loc_code *attn_loc, int new_state)
 	int	rc;
 	int	state;
 
-	rc = get_indicator_state(ATTN_INDICATOR, attn_loc, &state);
+	rc = get_indicator_state(LED_TYPE_FAULT, attn_loc, &state);
 	if (rc || state != new_state) {
-		rc = set_indicator_state(ATTN_INDICATOR, attn_loc, new_state);
+		rc = set_indicator_state(LED_TYPE_FAULT, attn_loc, new_state);
 
 		if (rc)
 			indicator_log_write("System Attention Indicator :"
@@ -109,9 +122,9 @@ set_fault_indicator(struct loc_code *loc, int new_state)
 	int	rc;
 	int	state;
 
-	rc = get_indicator_state(ATTN_INDICATOR, loc, &state);
+	rc = get_indicator_state(LED_TYPE_FAULT, loc, &state);
 	if (rc || state != new_state) {
-		rc = set_indicator_state(ATTN_INDICATOR, loc, new_state);
+		rc = set_indicator_state(LED_TYPE_FAULT, loc, new_state);
 
 		if (rc)
 			indicator_log_write("%s : Unable to %s fault indicator",
@@ -252,6 +265,7 @@ build_callout_loc_code(struct sl_event *event, struct loc_code *list,
 
 		/* get FRUs nearest fault indicator */
 		strncpy(location, callout->location, LOCATION_LENGTH);
+		location[LOCATION_LENGTH - 1] = '\0';
 		loc_led = get_fru_indicator(list, location, &truncated);
 		if (!loc_led) { /* No indicator found for the given loc code */
 			*attn_state = 1;
@@ -273,12 +287,10 @@ build_callout_loc_code(struct sl_event *event, struct loc_code *list,
 					fru = fru->next;
 
 			if (!fru) {
-				fru = (struct fru *)malloc(sizeof
-							   (struct fru));
+				fru = calloc(1, sizeof(struct fru));
 				frus = fru;
 			} else {
-				fru->next = (struct fru *)malloc(sizeof
-							       (struct fru));
+				fru->next = calloc(1, sizeof(struct fru));
 				fru = fru->next;
 			}
 			if (!fru) {
@@ -287,8 +299,7 @@ build_callout_loc_code(struct sl_event *event, struct loc_code *list,
 				return NULL;
 			}
 
-			memset(fru, 0, sizeof(struct fru));
-			strncpy(fru->location, location, LOCATION_LENGTH);
+			strncpy(fru->location, location, LOCATION_LENGTH - 1);
 		}
 	}
 
@@ -331,6 +342,7 @@ event_fru_callout(struct sl_callout *callouts, struct loc_code *list,
 
 		/* get FRUs nearest fault indicator */
 		strncpy(location, callout->location, LOCATION_LENGTH);
+		location[LOCATION_LENGTH - 1] = '\0';
 		loc_led = get_fru_indicator(list, location, &truncated);
 		if (!loc_led) { /* No indicator found for the given loc code */
 			*attn_on = 1;
@@ -348,7 +360,7 @@ event_fru_callout(struct sl_callout *callouts, struct loc_code *list,
 								location);
 		}
 
-		rc = set_fault_indicator(loc_led, INDICATOR_ON);
+		rc = set_fault_indicator(loc_led, LED_STATE_ON);
 	}
 	return rc;
 }
@@ -371,7 +383,7 @@ repair_fru_callout(struct fru *fru, struct loc_code *list, int *attn_disable)
 
 	loc_led = get_indicator_for_loc_code(list, fru->location);
 	if (loc_led)
-		rc = set_fault_indicator(loc_led, INDICATOR_OFF);
+		rc = set_fault_indicator(loc_led, LED_STATE_OFF);
 	else {
 		indicator_log_write("%s does not have fault indicator",
 				    fru->location);
@@ -419,7 +431,7 @@ parse_service_event(int event_id)
 	indicator_log_write("---- Service event begin ----");
 
 	/* build indicator list */
-	rc = get_indicator_list(ATTN_INDICATOR, &list);
+	rc = get_indicator_list(LED_TYPE_FAULT, &list);
 	if (rc) {
 		log_msg("Unable to retrieve fault indicators");
 		indicator_log_write("Unable to retrieve fault indicators");
@@ -429,7 +441,7 @@ parse_service_event(int event_id)
 	/* First loc code is system attention indicator */
 	attn_loc = &list[0];
 
-	if (operating_mode == LIGHT_PATH_MODE) {
+	if (operating_mode == LED_MODE_LIGHT_PATH) {
 		if (event->callouts)
 			/* Run over FRU callout priority in order and
 			 * enable fault indicator
@@ -445,10 +457,10 @@ parse_service_event(int event_id)
 		}
 
 		if (attn_on)	/* check log indicator */
-			rc = set_attn_indicator(attn_loc, INDICATOR_ON);
+			rc = set_attn_indicator(attn_loc, LED_STATE_ON);
 	} else {
 		log_msg("Guiding Light mode");
-		rc = set_attn_indicator(attn_loc, INDICATOR_ON);
+		rc = set_attn_indicator(attn_loc, LED_STATE_ON);
 	}
 
 	if (rc)
@@ -518,7 +530,7 @@ parse_repair_event(int repair_id)
 	indicator_log_write("---- Repair event begin ----");
 
 	/* build indicator list */
-	rc = get_indicator_list(ATTN_INDICATOR, &list);
+	rc = get_indicator_list(LED_TYPE_FAULT, &list);
 	if (rc) {
 		log_msg("Unable to retrieve fault indicators");
 		goto event_out;
@@ -527,7 +539,7 @@ parse_repair_event(int repair_id)
 	/* First loc code is system attention indicator */
 	attn_loc = &list[0];
 
-	if (operating_mode == LIGHT_PATH_MODE) {
+	if (operating_mode == LED_MODE_LIGHT_PATH) {
 		for_each_event(repair, repair_events) {
 			if (!service_event_supported(repair))
 				continue;
@@ -561,13 +573,13 @@ parse_repair_event(int repair_id)
 		}
 
 		if (!attn_keep && attn_disable)
-			rc = set_attn_indicator(attn_loc, INDICATOR_OFF);
+			rc = set_attn_indicator(attn_loc, LED_STATE_OFF);
 
 	} else {
 		log_msg("Guiding Light mode");
 
 		if (!open_events) /* No more open events */
-			rc = set_attn_indicator(attn_loc, INDICATOR_OFF);
+			rc = set_attn_indicator(attn_loc, LED_STATE_OFF);
 	}
 
 	if (rc)
@@ -631,8 +643,11 @@ UI_help(MENU *my_menu)
 	cur = current_item(my_menu);
 	desc = item_description(cur);
 
-	if (!desc)
+	if (!desc) {
+		delwin(my_text_win);
+		delwin(my_help_win);
 		return;
+	}
 
 	if (!strcmp(desc, "ident"))
 		mvwprintw(my_text_win, 0, 0, help1);
@@ -666,7 +681,7 @@ UI_make_selection(WINDOW *my_menu_win, MENU *my_menu)
 {
 	int	index;
 	int	i;
-	int	x, y;
+	int	__attribute__((__unused__)) x, y;
 	ITEM	*cur;
 	const	char *desc = NULL;
 
@@ -743,8 +758,7 @@ UI_commit(WINDOW *my_menu_win, MENU *my_menu,
 		if (!strcmp(desc, "loc")) {
 			name = (char *)item_name(items[i]);
 			loc = get_indicator_for_loc_code(ident_list, name);
-			rc = get_indicator_state(IDENT_INDICATOR,
-						 loc, &ident);
+			rc = get_indicator_state(LED_TYPE_IDENT, loc, &ident);
 			if (rc) {
 				if (cur_state[i] == '+') {
 					cur_state[i] = prev_state[i];
@@ -759,14 +773,14 @@ UI_commit(WINDOW *my_menu_win, MENU *my_menu,
 			wrefresh(my_help_win);
 
 			if (!strcmp(desc, "attn")) {
-				rc = set_attn_indicator(&attn_list[0], INDICATOR_OFF);
+				rc = set_attn_indicator(&attn_list[0], LED_STATE_OFF);
 				cur_state[i] = ' ';
 
 				if (rc)
 					err = 1;
 			} else {
-				rc = set_indicator_state(IDENT_INDICATOR, loc,
-							 ident ? INDICATOR_OFF : INDICATOR_ON);
+				rc = set_indicator_state(LED_TYPE_IDENT, loc,
+							 ident ? LED_STATE_OFF : LED_STATE_ON);
 
 				if (rc) {
 					err = 1;
@@ -822,7 +836,7 @@ UI_update_indicator(WINDOW *my_menu_win, MENU *my_menu)
 {
 	int	index;
 	int	i, j;
-	int	x, y;
+	int	__attribute__((__unused__)) x, y;
 	int	count;
 	ITEM	*cur;
 
@@ -894,7 +908,7 @@ void resize_handler(int sig)
 static int
 create_menu(struct loc_code *ident_list, struct loc_code *attn_list)
 {
-	int	x, y;
+	int	__attribute__((__unused__)) x, y;
 	int	i = 0;
 	int	c;
 	int	rc;
@@ -918,13 +932,14 @@ create_menu(struct loc_code *ident_list, struct loc_code *attn_list)
 	else
 		length += 2;
 
-	cur_state = (char *)malloc(length);
-	prev_state = (char *)malloc(length);
+	cur_state = malloc(length);
+	prev_state = malloc(length);
 
 	my_items = (ITEM **)calloc(length, sizeof(ITEM *));
 
 	if (!cur_state || !prev_state || !my_items) {
 		log_msg("Out of memory");
+		free(my_items);
 		return -1;
 	}
 
@@ -947,9 +962,9 @@ create_menu(struct loc_code *ident_list, struct loc_code *attn_list)
 		i++;
 	}
 
-	for (cur = ident_list, i; cur; cur = cur->next, i++) {
+	for (cur = ident_list; cur; cur = cur->next, i++) {
 		/* Determine the identify indicator state */
-		rc = get_indicator_state(IDENT_INDICATOR, cur, &ident);
+		rc = get_indicator_state(LED_TYPE_IDENT, cur, &ident);
 
 		if (rc)
 			cur_state[i] = ' ';
@@ -973,10 +988,16 @@ create_menu(struct loc_code *ident_list, struct loc_code *attn_list)
 	mvprintw(2, 0, "Make selection(s), use Commit to continue");
 	mvprintw(LINES - 3, 0, "F1=Help\t\t\tF4=List\t\t\tF7=Commit\t\t\t"
 					"F10=Exit\nF3=Previous Menu");
+	refresh();
+
 	my_menu = new_menu((ITEM **)my_items);
 	menu_opts_off(my_menu, O_SHOWDESC);
 	menu_opts_off(my_menu, O_ONEVALUE);
+
+	/* Set main window and sub window */
 	my_menu_win = newwin(nlines, 70, BEGIN_Y , 1);
+	set_menu_sub(my_menu, derwin(my_menu_win, nlines, 70, 0, 0));
+
 	keypad(my_menu_win, TRUE);
 	set_menu_win(my_menu, my_menu_win);
 	set_menu_format(my_menu, nlines, 1);
@@ -1074,12 +1095,12 @@ startup_window(void)
 		return -1;
 	}
 
-	rc = get_indicator_list(ATTN_INDICATOR, &attn_list);
+	rc = get_indicator_list(LED_TYPE_FAULT, &attn_list);
 
 	if (rc)
 		log_msg("Unable to get system attention indicator");
 
-	rv = get_indicator_list(IDENT_INDICATOR, &ident_list);
+	rv = get_indicator_list(LED_TYPE_IDENT, &ident_list);
 
 	if (rv)
 		log_msg("Unable to get identify indicators");
@@ -1147,15 +1168,11 @@ main(int argc, char *argv[])
 	int	repair_flag = 0;
 	int	event_id = 0;
 	int	repair_id = 0;
-	int	platform = 0;
 	char	*next_char;
 
-	platform = get_platform();
-	if (platform != PLATFORM_PSERIES_LPAR) {
-		fprintf(stderr, "%s is not supported on the %s platform\n",
-				argv[0], __power_platform_name(platform));
+	program_name = argv[0];
+	if (probe_indicator() != 0)
 		return 1;
-	}
 
 	opterr = 0;
 	while ((c = getopt_long(argc, argv, LP_DIAG_ARGS,
@@ -1211,13 +1228,12 @@ main(int argc, char *argv[])
 	}
 
 	/* initialize */
-	program_name = argv[0];
 	rc = init_files();
 	if (rc)
 		goto cleanup;
 
 	/* Light Path operating mode */
-	rc = check_operating_mode();
+	rc = get_indicator_mode();
 	if (rc)
 		goto cleanup;
 
