@@ -3,7 +3,20 @@
  * @brief	usysident/usysattn/usysfault
  *
  * Copyright (C) 2012 IBM Corporation
- * See 'COPYRIGHT' for License of this code.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * @author	Vasant Hegde <hegdevasant@linux.vnet.ibm.com>
  *
@@ -23,8 +36,8 @@
 #include <string.h>
 #include <stdint.h>
 
-#include "platform.h"
-#include "lp_diag.h"
+#include "indicator.h"
+#include "lp_util.h"
 
 #define CMD_LINE_OPTIONS	"d:l:s:-:thV"
 #define CMD_IDENTIFY		"usysident"
@@ -37,14 +50,13 @@
 
 /* Log indicator status */
 #define indicator_log_state(indicator, loc, state) \
-	if (indicator == ATTN_INDICATOR && !strchr(loc, '-')) \
+	if (indicator == LED_TYPE_FAULT && !strchr(loc, '-')) \
 		indicator_log_write("System Attention Indicator : %s", \
-				    state == INDICATOR_ON ? "ON" : "OFF"); \
+				    state == LED_STATE_ON ? "ON" : "OFF"); \
 	else \
 		indicator_log_write("%s : %s : %s", loc, \
-				    indicator == IDENT_INDICATOR ? \
-				    "Identify" : "Fault", \
-				    state == INDICATOR_ON ? "ON" : "OFF");
+				    get_indicator_desc(LED_TYPE_IDENT), \
+				    state == LED_STATE_ON ? "ON" : "OFF");
 
 
 /**
@@ -89,12 +101,10 @@ main(int argc, char **argv)
 {
 	int	c;
 	int	state;
-	int	index;
 	int	indicator;
 	int	rc = 0;
 	int	trunc = 0;
 	int	truncated = 0;
-	int	platform = 0;
 	char	temp[LOCATION_LENGTH];
 	char	dloc[LOCATION_LENGTH];
 	char	*dvalue = NULL;
@@ -104,12 +114,9 @@ main(int argc, char **argv)
 	struct	loc_code *current;
 	struct	loc_code *list = NULL;
 
-	platform = get_platform();
-	if (platform != PLATFORM_PSERIES_LPAR) {
-		fprintf(stderr, "%s is not supported on the %s platform\n",
-				argv[0], __power_platform_name(platform));
+	program_name = argv[0];
+	if (probe_indicator() != 0)
 		return 1;
-	}
 
 	opterr = 0;
 	while ((c = getopt(argc, argv, CMD_LINE_OPTIONS)) != -1) {
@@ -165,9 +172,9 @@ main(int argc, char **argv)
 	}
 
 	/* Option checking */
-	for (index = optind; index < argc; index++) {
+	if (optind < argc) {
 		fprintf(stderr,
-			"Unrecognized argument : %s\n\n", argv[index]);
+			"Unrecognized argument : %s\n\n", argv[optind]);
 		print_usage(argv[0]);
 		return 1;
 	}
@@ -187,9 +194,9 @@ main(int argc, char **argv)
 
 	if (svalue && strstr(argv[0], CMD_IDENTIFY)) {
 		if (!strcmp(svalue, "identify"))
-			c = INDICATOR_ON;
+			c = LED_STATE_ON;
 		else if (!strcmp(svalue, "normal"))
-			c = INDICATOR_OFF;
+			c = LED_STATE_OFF;
 		else {
 			fprintf(stderr,
 				"The -s option must be either "
@@ -202,7 +209,7 @@ main(int argc, char **argv)
 	if (svalue && (strstr(argv[0], CMD_FAULT) ||
 		       strstr(argv[0], CMD_ATTN))) {
 		if (!strcmp(svalue, "normal"))
-			c = INDICATOR_OFF;
+			c = LED_STATE_OFF;
 		else {
 			fprintf(stderr,
 				"The -s option must be \"normal\".\n\n");
@@ -246,9 +253,9 @@ main(int argc, char **argv)
 
 	if (othervalue && strstr(argv[0], CMD_IDENTIFY)) {
 		if (!strcmp(othervalue, "all-on"))
-			c = INDICATOR_ON;
+			c = LED_STATE_ON;
 		else if (!strcmp(othervalue, "all-off"))
-			c = INDICATOR_OFF;
+			c = LED_STATE_OFF;
 		else {
 			fprintf(stderr,
 				"Unrecognized option: --%s\n\n", othervalue);
@@ -260,7 +267,7 @@ main(int argc, char **argv)
 	if (othervalue && (strstr(argv[0], CMD_ATTN) ||
 			   strstr(argv[0], CMD_FAULT))) {
 		if (!strcmp(othervalue, "all-off"))
-			c = INDICATOR_OFF;
+			c = LED_STATE_OFF;
 		else {
 			fprintf(stderr,
 				"Unrecognized option: --%s\n\n", othervalue);
@@ -278,14 +285,13 @@ main(int argc, char **argv)
 	}
 
 	if (strstr(argv[0], CMD_IDENTIFY))
-		indicator = IDENT_INDICATOR;
+		indicator = LED_TYPE_IDENT;
 	else if (strstr(argv[0], CMD_FAULT) || strstr(argv[0], CMD_ATTN))
-		indicator = ATTN_INDICATOR;
+		indicator = LED_TYPE_FAULT;
 	else
 		return 1;
 
 	/* initialize */
-	program_name = argv[0];
 	lp_error_log_fd = 1; /* log message to stdout */
 	rc = init_files();
 	if (rc) {
@@ -294,8 +300,8 @@ main(int argc, char **argv)
 	}
 
 	/* Light Path operating mode */
-	if (indicator == ATTN_INDICATOR) {
-		rc = check_operating_mode();
+	if (indicator == LED_TYPE_FAULT) {
+		rc = get_indicator_mode();
 		if (rc)
 			return rc;
 	}
@@ -349,7 +355,8 @@ retry:
 				}
 			}
 			fprintf(stdout, "There is no %s indicator at location "
-				"code %s.\n", INDICATOR_TYPE(indicator), temp);
+				"code %s.\n",
+				get_indicator_desc(indicator), temp);
 			rc = 1;
 		} else { /* Found location code */
 			if (truncated)
@@ -410,12 +417,12 @@ retry:
 		 * then turning OFF all components ident indicator inside
 		 * enclosure does not turn OFF enclosure ident indicator.
 		 */
-		if (indicator == IDENT_INDICATOR && c == INDICATOR_OFF)
+		if (indicator == LED_TYPE_IDENT && c == LED_STATE_OFF)
 			set_indicator_state(indicator, &list[0], c);
 
 		indicator_log_write("All %s Indicators : %s",
-				    indicator == IDENT_INDICATOR ? "Identify" : "Fault",
-				    c == INDICATOR_ON ? "ON" : "OFF");
+				    get_indicator_desc(indicator),
+				    c == LED_STATE_ON ? "ON" : "OFF");
 	}
 
 indicator_cleanup:
