@@ -1,6 +1,20 @@
 /*
  * @file opal_errd_parse.c
  * Copyright (C) 2014 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -69,12 +83,18 @@ void print_usage(char *command)
 static int file_filter(const struct dirent *d)
 {
 	struct stat sbuf;
+	char filename[PATH_MAX];
+
 	if (d->d_type == DT_DIR)
 		return 0;
 	if (d->d_type == DT_REG)
 		return 1;
-	if (stat(d->d_name,&sbuf))
+
+	snprintf(filename, PATH_MAX, "%s/%s", opt_platform_dir, d->d_name);
+	if (stat(filename, &sbuf))
 		return 0;
+	if (S_ISREG(sbuf.st_mode))
+		return 1;
 	if (S_ISDIR(sbuf.st_mode))
 		return 0;
 	if (d->d_type == DT_UNKNOWN)
@@ -329,10 +349,9 @@ int elog_summary(char *elog_path, uint32_t service_flag)
 	printf("|------------------------------------------------------------------------------|\n");
 
 	sz = read_elog(elog_path, &buffer);
-
-	if (sz <= 0){
+	if (sz < 0)
 		return -1;
-	}
+
 	if (sz < ELOG_MIN_READ_OFFSET) {
 		fprintf(stderr, "Partially read elog, cannot parse\n");
 		ret = -1;
@@ -340,19 +359,16 @@ int elog_summary(char *elog_path, uint32_t service_flag)
 		print_elog_summary(buffer, sz, service_flag);
 	}
 
-	free(buffer);
-
-	if(!ret){
+	if (!ret)
 		printf("|------------------------------------------------------------------------------|\n");
-	}
 
+	free(buffer);
 	return ret;
 }
 
 /* list all the error logs */
 int eloglist(uint32_t service_flag)
 {
-	int ret = 0;
 	char *buffer;
 	struct dirent **filelist;
 	int nfiles;
@@ -377,8 +393,7 @@ int eloglist(uint32_t service_flag)
 
 	for (i = 0; i < nfiles; i++){
 		sz = read_elog(filelist[i]->d_name, &buffer);
-
-		if (sz <= 0){
+		if (sz < 0){
 			free(filelist[i]);
 			continue;
 		} else if (sz < ELOG_MIN_READ_OFFSET) {
@@ -392,11 +407,9 @@ int eloglist(uint32_t service_flag)
 	}
 	free(filelist);
 
-	if(!ret){
-		printf("|------------------------------------------------------------------------------|\n");
-	}
+	printf("|------------------------------------------------------------------------------|\n");
 
-	return ret;
+	return 0;
 }
 
 int delete_elog(const char *eid)
