@@ -3,6 +3,20 @@
  * @brief Routines for parsing the ppc64-diag.config file
  *
  * Copyright (C) 2004 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -11,7 +25,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <time.h>
-#include <stdarg.h> 
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
@@ -23,7 +37,7 @@
 #define RTAS_PARAM_AUTO_RESTART		21
 
 /**
- * @var config_file 
+ * @var config_file
  * @brief default ppc64-diag configuration file
  */
 char *config_file = "/etc/ppc64-diag/ppc64-diag.config";
@@ -79,7 +93,7 @@ get_token(char *str, char *str_end, char *tok, int *line_no)
 			 * started then skip ahead to the next line and
 			 * return that point as the end of the search.
 			 */
-			snprintf(tok, str - start + 1, start);
+			snprintf(tok, str - start + 1, "%s", start);
 			
 			while ((str < str_end) && (*str++ != '\n'));
 			(*line_no)++;
@@ -97,7 +111,7 @@ get_token(char *str, char *str_end, char *tok, int *line_no)
 				break;
 			}
 
-			snprintf(tok, str - start + 1, start);
+			snprintf(tok, str - start + 1, "%s", start);
 			return str + 1;
 			break;
 
@@ -112,7 +126,7 @@ get_token(char *str, char *str_end, char *tok, int *line_no)
 				(*line_no)++;
 				str++;
 			} else {
-				snprintf(tok, str - start + 1, start);
+				snprintf(tok, str - start + 1, "%s", start);
 			}
 
 			return str;
@@ -126,10 +140,10 @@ get_token(char *str, char *str_end, char *tok, int *line_no)
 			 * a token on their own.
 			 */
 			if (start == NULL) {
-				snprintf(tok, 2, str);
+				snprintf(tok, 2, "%s", str);
 				str++;
 			} else {
-				snprintf(tok, str - start + 1, start);
+				snprintf(tok, str - start + 1, "%s", start);
 			}
 
 			return str;
@@ -150,10 +164,10 @@ get_token(char *str, char *str_end, char *tok, int *line_no)
 					else
 						str++;
 				}
-				snprintf(tok, str - start + 1, start);
+				snprintf(tok, str - start + 1, "%s", start);
 				str++;
 			} else {
-				snprintf(tok, str - start + 1, start);
+				snprintf(tok, str - start + 1, "%s", start);
 			}
 
 			return str;
@@ -210,7 +224,7 @@ get_config_string(char *start, char *end, char *buf, int *line_no)
 	if (start == NULL)
 		return NULL;
 
-	offset += sprintf(buf, tok);
+	offset += sprintf(buf, "%s", tok);
 
 	start = get_token(start, end, tok, line_no);
 	while ((start != NULL) && (tok[0] != '\n')) {
@@ -300,9 +314,10 @@ config_restart_policy(char *start, char *end, int *line_no, int update_param)
 {
 	struct stat	sbuf;
 	char		*cur;
-	char		system_arg[80];
 	char		param[3];
-	int		rc;
+	pid_t		cpid;		/* Pid of child */
+	int		rc;		/* Holds return value */
+	int		status;		/* exit value of child */
 
 	cur = get_restart_policy_value(start, end,
 				       &d_cfg.restart_policy, line_no);
@@ -328,7 +343,7 @@ config_restart_policy(char *start, char *end, int *line_no, int update_param)
 		return cur;
 
 	/* Try to set the system parameter with the ibm,set-sysparm RTAS call */
-	*(uint16_t *)param = 1;
+	*(uint16_t *)param = htobe16(1);
 	param[2] = (uint8_t)d_cfg.restart_policy;
 	rc = rtas_set_sysparm(RTAS_PARAM_AUTO_RESTART, param);
 	switch (rc) {
@@ -372,26 +387,79 @@ config_restart_policy(char *start, char *end, int *line_no, int update_param)
 	/* See if the ibm,os-auto-restart config variable is in 
 	 * nvram on this machine
 	 */
-	sprintf(system_arg, "/usr/sbin/nvram "
-		"--print-config=ibm,os-auto-restart");
-	if (system(system_arg) == -1) {
+	cpid = fork();
+	if (cpid == -1) {
+		d_cfg.log_msg("Fork failed, while collecting ibm,os-auto-restart\n");
+		d_cfg.restart_policy = -1;
+		return cur;
+	} /* fork */
+
+	if (cpid == 0) { /* child */
+		char *system_arg[3]= {NULL, };
+
+		system_arg[0] = "/usr/sbin/nvram";
+		system_arg[1] = "--print-config=ibm,os-auto-restart";
+
+		rc = execv(system_arg[0], system_arg);
 		d_cfg.log_msg("The current system does not support the "
 			      "Auto Restart Policy; the AutoRestartPolicy "
 			      "configuration entry (line %d) is being skipped",
 			      line_no);
 		d_cfg.restart_policy = -1;
-		return cur;
+		exit(-2);
+	} else {/* parent */
+		/*
+		 * Use wait to make sure we do not update ibm,os-restart-policy,
+		 * before reading ibm,os-auto-restart.
+		 */
+		rc = waitpid(cpid, &status, 0);
+		if (rc == -1) {
+			d_cfg.log_msg("wait failed, while collecting"
+				      " ibm,os-auto-restart\n");
+			d_cfg.restart_policy = -1;
+			return cur;
+		}
+
+		if ((signed char)WEXITSTATUS(status) == -2)
+			return cur;
 	}
 
-	sprintf(system_arg, "%s%d", "/usr/sbin/nvram -p common "
-		"--update-config ibm,os-restart-policy=", d_cfg.restart_policy);
+	cpid = fork();
+	if (cpid == -1) {
+		d_cfg.log_msg("Fork failed, on updating ibm,os-restart-policy\n");
+		d_cfg.restart_policy = -1;
+		return cur;
+	} /* fork */
 
-	if (system(system_arg)) {
+	if (cpid == 0) { /* child */
+		char *system_arg[6]= {NULL,};
+		char tmp_sys_arg[50];
+
+		system_arg[0] = "/usr/sbin/nvram";
+		system_arg[1] = "-p";
+		system_arg[2] = "common";
+		system_arg[3] = "--update-config";
+		sprintf(tmp_sys_arg, "ibm,os-restart-policy=%d",
+			d_cfg.restart_policy);
+		system_arg[4] = tmp_sys_arg;
+
+		rc = execv(system_arg[0], system_arg);
 		d_cfg.log_msg("The Auto Restart Policy could not be configured "
 			      "due to a failure in running the nvram command",
 			      d_cfg.restart_policy);
 		d_cfg.restart_policy = -1;
-		return cur;
+		exit(-2);
+	} else { /* parent  */
+		rc = waitpid(cpid, &status, 0);
+		if (rc == -1) {
+			d_cfg.log_msg("wait failed, on updating ibm,os-restart-policy\n");
+			d_cfg.restart_policy = -1;
+			return cur;
+		}
+
+		/* Return on EXIT_FAILURE */
+		if ((signed char)WEXITSTATUS(status) == -2)
+			return cur;
 	}
 
 	d_cfg.log_msg("Configuring the Auto Restart Policy to %d (in NVRAM)",
