@@ -3,6 +3,20 @@
  * @brief Command to extract a platform dump and copy it to the filesystem
  *
  * Copyright (C) 2007 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -359,7 +373,7 @@ main(int argc, char *argv[])
 	platform = get_platform();
 	switch (platform) {
 	case PLATFORM_UNKNOWN:
-	case PLATFORM_POWERKVM:
+	case PLATFORM_POWERNV:
 		fprintf(stderr, "%s: is not supported on the %s platform\n",
 				argv[0], __power_platform_name(platform));
 		return -1;
