@@ -3,6 +3,20 @@
  * @brief Routines to handle CPU guard RTAS events
  *
  * Copyright (C) 2004 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -15,6 +29,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <dirent.h>
+#include "utils.h"
 #include "rtas_errd.h"
 
 #define DRMGR_PROGRAM		"/usr/sbin/drmgr"
@@ -185,8 +200,10 @@ retrieve_drc_name(enum event_type type, struct event *event, unsigned int id,
 	struct stat sbuf;
 	int rc, ret=1;
 	FILE *fp;
-	char cmd[1024];
 	uint8_t status;
+	char *system_args[9] = {NULL,}; /* execv arguments      	*/
+	char tmp_sys_arg[100];		/* tmp sys_arg for snprintf	*/
+	pid_t cpid;                     /* child pid            	*/
 
 	if (stat(CONVERT_DT_PROPS_PROGRAM, &sbuf) < 0) {
 		log_msg(event, "The command \"%s\" does not exist, %s",
@@ -194,14 +211,24 @@ retrieve_drc_name(enum event_type type, struct event *event, unsigned int id,
 		return 0;
 	}
 
+	system_args[0] = CONVERT_DT_PROPS_PROGRAM;
+	system_args[1] = "--context";
 	if (type == CPUTYPE) {
-		snprintf(cmd, 1024,"%s --context cpu --from interrupt-server "
-		"--to drc-name %u 2>/dev/null", CONVERT_DT_PROPS_PROGRAM, id);
-	}
-	else { /* event type is mem */
-		snprintf(cmd, 1024, "%s --context mem --from drc-index --to "
-			 "drc-name 0x%08x 2>/dev/null", CONVERT_DT_PROPS_PROGRAM,
-			  id);
+		system_args[2] = "cpu";
+		system_args[3] = "--from";
+		system_args[4] = "interrupt-server";
+		system_args[5] = "--to";
+		system_args[6] = "drc-name";
+		snprintf(tmp_sys_arg, 100, "%u", id);
+		system_args[7] = tmp_sys_arg;
+	} else { /* event type is mem */
+		system_args[2] = "mem";
+		system_args[3] = "--from";
+		system_args[4] = "drc-index";
+		system_args[5] = "--to";
+		system_args[6] = "drc-name";
+		snprintf(tmp_sys_arg, 100, "0x%08x", id);
+		system_args[7] = tmp_sys_arg;
 	}
 
 	/* Note :
@@ -215,7 +242,8 @@ retrieve_drc_name(enum event_type type, struct event *event, unsigned int id,
 	 */
 	restore_sigchld_default();
 
-	if ((fp = popen(cmd, "r")) == NULL) {
+	fp = spopen(system_args, &cpid);
+	if (fp == NULL) {
 		if (type == CPUTYPE) {
 			log_msg(event, "Cannot obtain the drc-name for the "
 				"CPU with ID %u; Could not run %s. %s", id,
@@ -228,42 +256,31 @@ retrieve_drc_name(enum event_type type, struct event *event, unsigned int id,
 		}
 		setup_sigchld_handler();
 		return 0;
-	}
+	} /* fp == NULL */
 
 	rc = fread(buffer, 1, bufsize, fp);
 	if (rc == bufsize) {
-		if (type == CPUTYPE) {
-			log_msg(event, "Cannot obtain the drc-name for the "
-				       "CPU with ID %u; Buffer overflow in "
-				       "retrieve_drc_name", id);
-		}
-		else {
-			log_msg(event, "Cannot obtain the drc-name for the "
-				       "MEMORY with ID %u; Buffer overflow "
-				       "in retrieve_drc_name", id);
-		}
+		log_msg(event, "Cannot obtain the drc-name for the "
+			       "%s with ID %u; Buffer overflow in "
+			       "retrieve_drc_name",
+				(type == CPUTYPE) ? "CPU" : "MEMORY",
+				id);
 		buffer[bufsize-1] = '\0';
 		ret = 0;
-	}
-	else {
+	} else if (rc > 0) {
 		buffer[rc-1] = '\0';	/* overwrite the newline at the end */
-	}
+	} else
+		buffer[0] = '\0';
 
-	rc = pclose(fp);
-	status = (int8_t)WEXITSTATUS(rc);
+	status = spclose(fp, cpid);
 
 	setup_sigchld_handler();
 
 	if (status != 0) {
-		if (type == CPUTYPE) {
-			log_msg(event, "Cannot obtain the drc-name for the "
-				       "CPU with ID %u; %s returned %d",
-					id, CONVERT_DT_PROPS_PROGRAM, rc);
-		}
-		else {
-			log_msg(event, "Cannot obtain the drc-name for the "
-					"Memory with ID %u; %s returned %d", 						id, CONVERT_DT_PROPS_PROGRAM, rc);
-		}
+		log_msg(event, "Cannot obtain the drc-name for the "
+			       "%s with ID %u; %s returned %d",
+				(type == CPUTYPE) ? "CPU" : "MEMORY",
+				id, CONVERT_DT_PROPS_PROGRAM, rc);
 		ret = 0;
 	}
 
@@ -294,8 +311,10 @@ parse_lparcfg(char *param) {
 		if (!strncmp(param, buffer, len)) {
 			if ((pos = strchr(buffer, '=')) == NULL) {
 				log_msg(NULL, "Could not retrieve the value "
-					"for %s from /proc/ppc64/lparcfg", 
+					"for %s from /proc/ppc64/lparcfg",
 					param);
+				fclose(fp);
+				return -1;
 			}
 			pos++;
 			buffer[strlen(buffer)-1] = '\0';
