@@ -3,6 +3,20 @@
  * @brief Main entry point for rtas_errd.
  *
  * Copyright (C) 2004 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -115,7 +129,7 @@ handle_rtas_event(struct event *event)
 	 * be done before the error log is written to disk, because
 	 * the log will be updated with the path to the dump
 	 */
-	dbg("Entering check_patform_dump()");
+	dbg("Entering check_platform_dump()");
 	check_platform_dump(event);
 
 	/* write the event to the platform file */
@@ -139,7 +153,7 @@ handle_rtas_event(struct event *event)
 	    case RTAS_HDR_TYPE_EPOW:
 		dbg("Entering check_epow()");
 		if (check_epow(event) <= 0) {
-			dbg("Recevied EPOW 0 (all is normal) event");
+			dbg("Received EPOW 0 (all is normal) event");
 			return 0;
 		}
 		break;
@@ -220,7 +234,7 @@ handle_rtas_event(struct event *event)
  * read_rtas_event
  * @brief Main routine to retrieve RTAS events from the kernel
  * 
- * Responsable for reading RTAS events from the kernel (via /proc)
+ * Responsible for reading RTAS events from the kernel (via /proc)
  * and calling handle_rtas_event() to process the event.
  */
 int
@@ -420,7 +434,7 @@ main(int argc, char *argv[])
 	platform = get_platform();
 	switch (platform) {
 	case PLATFORM_UNKNOWN:
-	case PLATFORM_POWERKVM:
+	case PLATFORM_POWERNV:
 		fprintf(stderr, "%s: is not supported on the %s platform\n",
 				argv[0], __power_platform_name(platform));
 		return -1;
@@ -509,7 +523,7 @@ main(int argc, char *argv[])
 	sigact.sa_flags = SA_RESTART;
 	if (sigaction(SIGALRM, &sigact, NULL)) {
 		log_msg(NULL, "Could not initialize signal handler for "
-			"certian EPOW events (SIGALRM), %s", strerror(errno));
+			"certain EPOW events (SIGALRM), %s", strerror(errno));
 		goto error_out;
 	}
 
